==> kildclient_2.5.0.orig.tar.gz: kildclient-2.5.0/src/aliases.c <==
/* $Id: aliases.c,v 1.22 2006/01/29 18:58:09 ekalin Exp $ */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <kcconfig.h>
#endif

#include <libintl.h>
#include <locale.h>
#include <gtk/gtk.h>
#include <glade/glade.h>

#include "kildclient.h"
#include "perlscript.h"
#include "ansi.h"


/*************************
 * File global variables *
 *************************/


/***********************
 * Function prototypes *
 ***********************/



char *
substitute_aliases(World *world, const char *cmd)
{
  GSList *aliasptr;
  Alias  *alias;
  char   *result;
  int     i;

  if (world->disable_aliases) {
    return NULL;
  }

  aliasptr = world->aliases;
  i = 0;
  while (aliasptr) {
    alias = (Alias *) aliasptr->data;
    if (alias->enabled && (result = perl_substitute(world->perl_interpreter,
                                                    cmd,
                                                    alias->pattern_re,
                                                    alias->substitution,
                                                    alias->perl_eval))) {
      if (debug_matches) {
        fprintf(stderr,
                "**Alias %d '%s' matched for world %s\n"
                "  Command: %s\n"
                "  Pattern: %s\n"
                "  Substitution: %s\n"
                "  Eval as Perl: %d\n",
                i,
                alias->name ? alias->name : "",
                world->name,
                cmd,
                alias->pattern,
                alias->substitution,
                alias->perl_eval);
      }

      return result;
    }
    aliasptr = aliasptr->next;
    ++i;
  }

  return NULL;
}


void
remove_alias(World *world, GSList *aliasitem)
{
  Alias *alias = (Alias *) aliasitem->data;

  we_alias_delete_alias(world, alias);

  world->aliases = g_slist_remove_link(world->aliases, aliasitem);
  if (!alias->owner_plugin) {
    --world->alias_pos;
  }
  free_alias(alias, NULL);
  g_slist_free(aliasitem);
}


gboolean
move_alias(World *world, gint old_pos, gint new_pos)
{
  GSList   *aliasitem;
  gpointer  alias;

  aliasitem = g_slist_nth(world->aliases, old_pos);
  if (!aliasitem) {
    return FALSE;
  }
  alias = aliasitem->data;

  world->aliases = g_slist_delete_link(world->aliases, aliasitem);
  world->aliases = g_slist_insert(world->aliases, alias, new_pos);

  we_alias_delete_alias(world, alias);
  we_alias_insert_alias(world, alias, new_pos);

  return TRUE;
}


void
free_alias(Alias *alias, gpointer data)
{
  g_free(alias->name);
  g_free(alias->pattern);
  g_free(alias->substitution);

  g_free(alias);
}


void
list_aliases(World *world, Plugin *plugin)
{
  int     i;
  guint   total_width;
  guint   field_width;
  GSList *aliasptr;
  Alias  *alias;

  /* The rows argument is not used. */
  ansitextview_get_size(world->gui, &field_width, &total_width);

  /* If the screen is really narrow, we can do nothing. */
  if (total_width < 30) {
    total_width = 30;
  }
  field_width = (total_width - 18)/2;

  ansitextview_append_stringf(world->gui,
                              _("Num Eval Ena IgC %-*.*s %-*.*s\n"),
                              field_width, field_width, _("Pattern"),
                              field_width, field_width, _("Substitution"));
  ansitextview_append_string(world->gui, "--- ---- --- --- ");
  for (i = 0; i < field_width; ++i) {
    ansitextview_append_string(world->gui, "-");
  }
  ansitextview_append_string(world->gui, " ");
  for (i = 0; i < field_width; ++i) {
    ansitextview_append_string(world->gui, "-");
  }
  ansitextview_append_string(world->gui, "\n");

  i = 0;
  aliasptr = world->aliases;
  while (aliasptr) {
    alias = (Alias *) aliasptr->data;

    if (alias->owner_plugin == plugin) {
      ansitextview_append_stringf(world->gui,
                                  "%3d %-4.4s %-3.3s %-3.3s %-*.*s %-*.*s\n",
                                  i,
                                  alias->perl_eval   ? _("y") : _("n"),
                                  alias->enabled     ? _("y") : _("n"),
                                  alias->ignore_case ? _("y") : _("n"),
                                  field_width, field_width,
                                  alias->pattern,
                                  field_width, field_width,
                                  alias->substitution);
    }

    ++i;
    aliasptr = aliasptr->next;
  }
}


void
alias_precompute_res(World *world)
{
  GSList *aliasptr = world->aliases;
  Alias  *alias;

  while (aliasptr) {
    alias = (Alias *) aliasptr->data;

    /* Aliases loaded from a script file already have the precomputed RE */
    if (!alias->pattern_re) {
      alias->pattern_re = precompute_re(world->perl_interpreter,
                                        alias->pattern,
                                        alias->ignore_case);
    }

    aliasptr = aliasptr->next;
  }
}


void
save_alias(FILE *fp, Alias *alias)
{
  fprintf(fp, "    <alias ");
  if (alias->name) {
    fprintf_escaped(fp, "name=\"%s\" ", alias->name);
  }
  fprintf(fp, "perleval=\"%d\" enabled=\"%d\" ignorecase=\"%d\">\n",
          alias->perl_eval,
          alias->enabled,
          alias->ignore_case);
  fprintf_escaped(fp,
                  "      <pattern>%s</pattern>\n"
                  "      <substitution>%s</substitution>\n",
                  alias->pattern,
                  alias->substitution);
  fprintf(fp, "    </alias>\n");
}

==> kildclient_2.6.0.orig.tar.gz: kildclient-2.6.0/src/aliases.c <==
/* $Id: aliases.c,v 1.23 2007/08/16 18:02:01 ekalin Exp $ */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 */

#ifdef HAVE_CONFIG_H
#  include <kcconfig.h>
#endif

#include <libintl.h>
#include <locale.h>
#include <gtk/gtk.h>
#include <glade/glade.h>

#include "kildclient.h"
#include "perlscript.h"
#include "ansi.h"


/*************************
 * File global variables *
 *************************/


/***********************
 * Function prototypes *
 ***********************/



char *
substitute_aliases(World *world, const char *cmd)
{
  GSList *aliasptr;
  Alias  *alias;
  char   *result;
  int     i;

  if (world->disable_aliases) {
    return NULL;
  }

  aliasptr = world->aliases;
  i = 0;
  while (aliasptr) {
    alias = (Alias *) aliasptr->data;
    if (alias->enabled && (result = perl_substitute(world->perl_interpreter,
                                                    cmd,
                                                    alias->pattern_re,
                                                    alias->substitution,
                                                    alias->perl_eval))) {
      if (debug_matches) {
        fprintf(stderr,
                "**Alias %d '%s' matched for world %s\n"
                "  Command: %s\n"
                "  Pattern: %s\n"
                "  Substitution: %s\n"
                "  Eval as Perl: %d\n"
                "  Result: %s\n",
                i,
                alias->name ? alias->name : "",
                world->name,
                cmd,
                alias->pattern,
                alias->substitution,
                alias->perl_eval,
                result);
      }

      return result;
    }
    aliasptr = aliasptr->next;
    ++i;
  }

  return NULL;
}


void
remove_alias(World *world, GSList *aliasitem)
{
  Alias *alias = (Alias *) aliasitem->data;

  we_alias_delete_alias(world, alias);

  world->aliases = g_slist_remove_link(world->aliases, aliasitem);
  if (!alias->owner_plugin) {
    --world->alias_pos;
  }
  free_alias(alias, NULL);
  g_slist_free(aliasitem);
}


gboolean
move_alias(World *world, gint old_pos, gint new_pos)
{
  GSList   *aliasitem;
  gpointer  alias;

  aliasitem = g_slist_nth(world->aliases, old_pos);
  if (!aliasitem) {
    return FALSE;
  }
  alias = aliasitem->data;

  world->aliases = g_slist_delete_link(world->aliases, aliasitem);
  world->aliases = g_slist_insert(world->aliases, alias, new_pos);

  we_alias_delete_alias(world, alias);
  we_alias_insert_alias(world, alias, new_pos);

  return TRUE;
}


void
free_alias(Alias *alias, gpointer data)
{
  g_free(alias->name);
  g_free(alias->pattern);
  g_free(alias->substitution);

  g_free(alias);
}


void
list_aliases(World *world, Plugin *plugin)
{
  int     i;
  guint   total_width;
  guint   field_width;
  GSList *aliasptr;
  Alias  *alias;

  /* The rows argument is not used. */
  ansitextview_get_size(world->gui, &field_width, &total_width);

  /* If the screen is really narrow, we can do nothing. */
  if (total_width < 30) {
    total_width = 30;
  }
  field_width = (total_width - 18)/2;

  ansitextview_append_stringf(world->gui,
                              _("Num Eval Ena IgC %-*.*s %-*.*s\n"),
                              field_width, field_width, _("Pattern"),
                              field_width, field_width, _("Substitution"));
  ansitextview_append_string(world->gui, "--- ---- --- --- ");
  for (i = 0; i < field_width; ++i) {
    ansitextview_append_string(world->gui, "-");
  }
  ansitextview_append_string(world->gui, " ");
  for (i = 0; i < field_width; ++i) {
    ansitextview_append_string(world->gui, "-");
  }
  ansitextview_append_string(world->gui, "\n");

  i = 0;
  aliasptr = world->aliases;
  while (aliasptr) {
    alias = (Alias *) aliasptr->data;

    if (alias->owner_plugin == plugin) {
      ansitextview_append_stringf(world->gui,
                                  "%3d %-4.4s %-3.3s %-3.3s %-*.*s %-*.*s\n",
                                  i,
                                  alias->perl_eval   ? _("y") : _("n"),
                                  alias->enabled     ? _("y") : _("n"),
                                  alias->ignore_case ? _("y") : _("n"),
                                  field_width, field_width,
                                  alias->pattern,
                                  field_width, field_width,
                                  alias->substitution);
    }

    ++i;
    aliasptr = aliasptr->next;
  }
}


void
alias_precompute_res(World *world)
{
  GSList *aliasptr = world->aliases;
  Alias  *alias;

  while (aliasptr) {
    alias = (Alias *) aliasptr->data;

    /* Aliases loaded from a script file already have the precomputed RE */
    if (!alias->pattern_re) {
      alias->pattern_re = precompute_re(world->perl_interpreter,
                                        alias->pattern,
                                        alias->ignore_case);
    }

    aliasptr = aliasptr->next;
  }
}


void
save_alias(FILE *fp, Alias *alias)
{
  fprintf(fp, "    <alias ");
  if (alias->name) {
    fprintf_escaped(fp, "name=\"%s\" ", alias->name);
  }
  fprintf(fp, "perleval=\"%d\" enabled=\"%d\" ignorecase=\"%d\">\n",
          alias->perl_eval,
          alias->enabled,
          alias->ignore_case);
  fprintf_escaped(fp,
                  "      <pattern>%s</pattern>\n"
                  "      <substitution>%s</substitution>\n",
                  alias->pattern,
                  alias->substitution);
  fprintf(fp, "    </alias>\n");
}
